﻿local _G = _G
local MT = MacroToolkit
local CreateFrame, UIParent = CreateFrame, UIParent
local PlaySound, PlaySoundFile, UnitName, GetNumMacros = PlaySound, PlaySoundFile, UnitName, GetNumMacros
local CreateMacro, DeleteMacro, EditMacro, GetMacroInfo = CreateMacro, DeleteMacro, EditMacro, GetMacroInfo
local GameTooltip_Hide, GameTooltip = GameTooltip_Hide, GameTooltip
local string, collectgarbage, select, table = string, collectgarbage, select, table
local ipairs, tostring = ipairs, tostring
local L = MT.L
local AceGUI = MT.LS("AceGUI-3.0")

function MT:CreateMTFrame()
	local mtframe = CreateFrame("Frame", "MacroToolkitFrame", UIParent, "ButtonFrameTemplate")

	local function frameMouseUp()
		mtframe:StopMovingOrSizing()
		MT.db.profile.x = mtframe:GetLeft()
		MT.db.profile.y = mtframe:GetBottom()
	end

	mtframe:SetSize(638, 424)
	mtframe:SetMovable(true)
	mtframe:EnableMouse(true)
	mtframe:SetPoint("BOTTOMLEFT", MT.db.profile.x, MT.db.profile.y)
	mtframe:SetScript("OnMouseDown", function() mtframe:StartMoving() end)
	mtframe:SetScript("OnMouseUp", frameMouseUp)

	local mtfportrait = mtframe:CreateTexture(nil, "OVERLAY", nil, -1)
	mtfportrait:SetTexture("Interface\\MacroFrame\\MacroFrame-Icon")
	mtfportrait:SetSize(60, 60)
	mtfportrait:SetPoint("TOPLEFT", -5, 5)
	
	local mttitle = mtframe:CreateFontString(nil, "BORDER", "GameFontNormal")
	mttitle:SetText("Macro Toolkit")
	mttitle:SetPoint("TOP", 0, -5)
	
	local mtcustomflyout = CreateFrame("Button", "MacroToolkitFlyout", mtframe)
	mtcustomflyout:SetSize(32, 32)
	mtcustomflyout:SetPoint("TOPRIGHT", -4, -27)
	mtcustomflyout:SetScript("OnEnter",
		function(this)
			GameTooltip:SetOwner(this, "ANCHOR_RIGHT")
			GameTooltip:SetText(L["Show custom commands"])
		end)
	mtcustomflyout:SetScript("OnClick",
		function()
			if MT.showingcustom then
				MT.showcustom.frame:Hide()
				MT.showingcustom = nil
			else
				if not MT.showcustom then
					MT.showcustom = AceGUI:Create("InlineGroup")
					MT.showcustom.frame:Hide()
					MT.showcustom:SetWidth(120)
					MT.showcustom:SetPoint("TOPLEFT", mtcustomflyout, "TOPRIGHT", 6, 0)
					MT.showcustom.frame:SetScript("OnShow", function() PlaySound("igCharacterInfoOpen") end)
					MT.showcustom.frame:SetScript("OnHide", function() PlaySound("igCharacterInfoClose") end)
				end
				MT.showcustom:ReleaseChildren()
				for _, c in ipairs(MT.db.global.custom) do
					local l = AceGUI:Create("Label")
					l:SetText(MT.slash .. c.n)
					l:SetWidth(100)
					MT.showcustom:AddChild(l)
				end
				MT.showcustom.frame:Show()
				MT.showingcustom = true
			end
		end)
	mtcustomflyout:SetScript("OnLeave", function() GameTooltip:Hide() end)
	mtcustomflyout:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up")
	mtcustomflyout:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down")
	mtcustomflyout:SetDisabledTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Disabled")
	mtcustomflyout:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight", "ADD")

	local mtcustom = CreateFrame("Button", "MacroToolkitCustom", mtframe, "UIPanelButtonTemplate")
	mtcustom:SetText(_G.CUSTOM)
	mtcustom:SetSize(80, 22)
	mtcustom:SetPoint("TOPRIGHT", mtcustomflyout, "TOPLEFT", 0, -5)
	mtcustom:SetScript("OnClick",
		function()
			if MT.showingcustom then
				MT.showcustom.frame:Hide()
				MT.showingcustom = nil
			end
			StaticPopup_Show("MACROTOOLKIT_SLASHNAME")
		end)

	local mthbleft = mtframe:CreateTexture(nil, "ARTWORK")
	mthbleft:SetTexture("Interface\\ClassTrainerFrame\\UI-ClassTrainer-HorizontalBar")
	mthbleft:SetSize(556, 16)
	mthbleft:SetPoint("TOPLEFT", 2, -210)
	mthbleft:SetTexCoord(0, 1, 0, 0.25)

	local mthbright = mtframe:CreateTexture(nil, "ARTWORK")
	mthbright:SetTexture("Interface\\ClassTrainerFrame\\UI-ClassTrainer-HorizontalBar")
	mthbright:SetSize(75, 16)
	mthbright:SetPoint("LEFT", mthbleft, "RIGHT", 0, 0)
	mthbright:SetTexCoord(0, 0.29296875, 0.25, 0.5)

	local mtselbg = mtframe:CreateTexture("MacroToolkitSelBg", "ARTWORK")
	mtselbg:SetTexture("Interface\\Buttons\\UI-EmptySlot")
	mtselbg:SetSize(64, 64)
	mtselbg:SetPoint("TOPLEFT", 5, -218)

	local mtselname = mtframe:CreateFontString("MacroToolkitSelMacroName", "ARTWORK", "GameFontNormalLarge")
	mtselname:SetJustifyH("LEFT")
	mtselname:SetSize(256, 16)
	mtselname:SetPoint("TOPLEFT", mtselbg, "TOPRIGHT", -4, -10)
	
	local mtentermt = mtframe:CreateFontString("MacroToolkitEnterText", "ARTWORK", "GameFontHighlightSmall")
	mtentermt:SetText(_G.ENTER_MACRO_LABEL)
	mtentermt:SetPoint("TOPLEFT", mtselbg, "BOTTOMLEFT", 8, 3)

	local mtcharlt = mtframe:CreateFontString("MacroToolkitLimit", "ARTWORK", "GameFontHighlightSmall")
	mtcharlt:SetSize(0, 10)
	mtcharlt:SetPoint("BOTTOM", -165, 30)
	
	local mtselbutton = CreateFrame("CheckButton", "MacroToolkitSelMacroButton", mtframe, "MacroButtonTemplate")
	mtselbutton:SetID(0)
	mtselbutton:SetPoint("TOPLEFT", mtselbg, 14, -14)
	mtselbutton:SetScript("OnClick", function(this) this:SetChecked(nil); PickupMacro(mtframe.selectedMacro) end)

	local mtscroll = CreateFrame("ScrollFrame", "MacroToolkitButtonScroll", mtframe, "UIPanelScrollFrameTemplate")
	mtscroll:SetSize(294, 146)
	mtscroll:SetPoint("TOPLEFT", 12, -66)
	ScrollFrame_OnLoad(mtscroll)
	
	local mtscrolltop = mtscroll:CreateTexture(nil, "ARTWORK")
	mtscrolltop:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-ScrollBar")
	mtscrolltop:SetSize(31, 102)
	mtscrolltop:SetPoint("TOPLEFT", mtscroll, "TOPRIGHT", -2, 5)
	mtscrolltop:SetTexCoord(0, 0.484375, 0, 0.4)
	
	local mtscrollbot = mtscroll:CreateTexture(nil, "ARTWORK")
	mtscrollbot:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-ScrollBar")
	mtscrollbot:SetSize(31, 106)
	mtscrollbot:SetPoint("BOTTOMLEFT", mtscroll, "BOTTOMRIGHT", -2, -2)
	mtscrollbot:SetTexCoord(0.515625, 1, 0, 0.4140625)
	
	local mtscrollmid = mtscroll:CreateTexture(nil, "ARTWORK")
	mtscrollmid:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-ScrollBar")
	mtscrollmid:SetSize(31, 1)
	mtscrollmid:SetPoint("TOP", mtscrolltop, "BOTTOM")
	mtscrollmid:SetPoint("BOTTOM", mtscrollbot, "TOP")
	mtscrollmid:SetTexCoord(0, 0.484375, 0.75, 1)

	local mtscrollchild = CreateFrame("Frame", "MacroToolkitButtonContainer")
	mtscrollchild:SetSize(285, 10)
	mtscrollchild:SetPoint("TOPLEFT")
	MT:ContainerOnLoad(mtscrollchild)
	mtscroll:SetScrollChild(mtscrollchild)

	local mteditbutton = CreateFrame("Button", "MacroToolkitEdit", mtframe, "UIPanelButtonTemplate")
	mteditbutton:SetText(_G.CHANGE_MACRO_NAME_ICON)
	mteditbutton:SetFrameStrata("HIGH")
	mteditbutton:SetSize(170, 22)
	mteditbutton:SetPoint("TOPLEFT", mtselbg, 55, -30)
	mteditbutton:SetScript("OnClick", 
		function()
			MT:SaveMacro()
			MacroToolkitPopup.mode = "new"
			MacroToolkitPopup:Show()
		end)

	local mtmscroll = CreateFrame("ScrollFrame", "MacroToolkitScrollFrame", mtframe, "UIPanelScrollFrameTemplate")
	mtmscroll:SetSize(286, 85)
	mtmscroll:SetPoint("TOPLEFT", mtselbg, "BOTTOMLEFT", 11, -13)

	local function onverticalscroll(this, offset)
		local scrollbar1 = MacroToolkitScrollFrameScrollBar
		local scrollbar2 = MacroToolkitFauxScrollFrameScrollBar
		scrollbar1:SetValue(offset)
		scrollbar2:SetValue(offset)
		local mini1, maxi1 = scrollbar1:GetMinMaxValues()
		local mini2, maxi2 = scrollbar2:GetMinMaxValues()
		if offset == 0 then
			MacroToolkitScrollFrameScrollBarScrollUpButton:Disable()
			MacroToolkitFauxScrollFrameScrollBarScrollUpButton:Disable()
		else
			MacroToolkitScrollFrameScrollBarScrollUpButton:Enable()
			MacroToolkitFauxScrollFrameScrollBarScrollUpButton:Enable()
		end
		if scrollbar1:GetValue() - maxi1 == 0 then
			MacroToolkitScrollFrameScrollBarScrollDownButton:Disable()
			MacroToolkitFauxScrollFrameScrollBarScrollDownButton:Disable()
		else
			MacroToolkitScrollFrameScrollBarScrollDownButton:Enable()
			MacroToolkitFauxScrollFrameScrollBarScrollDownButton:Enable()
		end
	end

	mtmscroll:SetScript("OnVerticalScroll", onverticalscroll)
	mtmscroll:SetScript("OnMouseWheel",
		function(this, value, scrollBar)
			ScrollFrameTemplate_OnMouseWheel(MacroToolkitFauxScrollFrame, value)
			ScrollFrameTemplate_OnMouseWheel(this, value)
		end)
	local mtmscrollchild = CreateFrame("EditBox", "MacroToolkitText", mtmscroll)
	mtmscrollchild:SetMultiLine(true)
	mtmscrollchild:SetAutoFocus(false)
	mtmscrollchild:SetCountInvisibleLetters(true)
	mtmscrollchild:SetSize(286, 85)
	mtmscrollchild:SetScript("OnTextChanged",
		function(this, userinput)
			MT:UpdateCharLimit()
			if not userinput then return end
			mtframe.textChanged = 1
			if MacroToolkitPopup.mode == "new" then MacroToolkitPopup:Hide() end
			local m,e = MT:FormatMacro(this:GetText())
			MacroToolkitFauxText:SetText(m)
			MacroToolkitErrors:SetText(e)
			ScrollingEdit_OnTextChanged(this, this:GetParent())
			ScrollingEdit_OnTextChanged(MacroToolkitFauxText, MacroToolkitFauxScrollFrame)
		end)
	
	local function reformat(key)
		if key == "enter" then mtmscrollchild:Insert("\n") end
		local _, err = MT:FormatMacro(mtmscrollchild:GetText())
		MacroToolkitErrors:SetText(err)
	end

	local function oncursorchanged(this, x, y, w, h)
		this.cursorOffset = y
		this.cursorHeight = h
		this.handleCursorChange = true
		MacroToolkitFauxText.cursorOffset = y
		MacroToolkitFauxText.cursorHeight = h
		MacroToolkitFauxText.handleCursorChange = true
	end
	
	mtmscrollchild:SetScript("OnCursorChanged", oncursorchanged)
	mtmscrollchild:SetScript("OnEnterPressed", function(this) reformat("enter") end)
	mtmscrollchild:SetScript("OnSpacePressed", function(this) reformat("space") end)
	mtmscrollchild:SetScript("OnUpdate",
		function(this)
			ScrollingEdit_OnUpdate(this)
			ScrollingEdit_OnUpdate(MacroToolkitFauxText)
		end)
	mtmscrollchild:SetScript("OnEscapePressed", EditBox_ClearFocus)
	mtmscrollchild:SetFontObject("GameFontHighlightSmall")
	mtmscrollchild:SetScript("OnEditFocusGained", nil)
	mtmscroll:SetScrollChild(mtmscrollchild)
	
	local mtmfscroll = CreateFrame("ScrollFrame", "MacroToolkitFauxScrollFrame", mtframe, "UIPanelScrollFrameTemplate")
	mtmfscroll:SetSize(286, 85)
	mtmfscroll:SetPoint("TOPLEFT", mtselbg, "BOTTOMLEFT", 11, -13)
	
	local mtmfscrollchild = CreateFrame("EditBox", "MacroToolkitFauxText", mtmfscroll)
	mtmfscrollchild:SetMultiLine(true)
	mtmfscrollchild:SetAutoFocus(false)
	mtmfscrollchild:SetCountInvisibleLetters(true)
	mtmfscrollchild:SetSize(286, 85)
	mtmfscrollchild:SetScript("OnUpdate", nil)
	mtmfscrollchild:SetScript("OnTextChanged", nil)
	mtmfscrollchild:SetFontObject("GameFontHighlightSmall")
	mtmscrollchild:SetScript("OnEditFocusGained", function() mtmscrollchild:SetFocus() end)
	mtmfscroll:SetScrollChild(mtmfscrollchild)

	local mterrorbg = CreateFrame("Frame", "MacroToolkitErrorBg", mtframe)
	mterrorbg:SetSize(302, 95)
	mterrorbg:SetPoint("TOPLEFT", 328, -289)
	mterrorbg:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16, tileSize = 16, tile = true, insets = {left = 5, right = 5, top = 5, bottom = 5}})
	mterrorbg:SetBackdropBorderColor(_G.TOOLTIP_DEFAULT_COLOR.r, _G.TOOLTIP_DEFAULT_COLOR.g, _G.TOOLTIP_DEFAULT_COLOR.b)
	mterrorbg:SetBackdropColor(_G.TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, _G.TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, _G.TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
	
	local mterrorscroll = CreateFrame("ScrollFrame", "MacroToolkitErrorScrollFrame", mtframe, "UIPanelScrollFrameTemplate")
	mterrorscroll:SetSize(266, 85)
	mterrorscroll:SetPoint("TOPLEFT", mterrorbg, 10, -6)
	local mtescrollchild = CreateFrame("EditBox", "MacroToolkitErrors", mterrorscroll)
	mtescrollchild:SetMultiLine(true)
	mtescrollchild:SetAutoFocus(false)
	mtescrollchild:SetEnabled(false)
	mtescrollchild:SetSize(266, 85)
	mtescrollchild:SetScript("OnUpdate", function(this, elapsed) ScrollingEdit_OnUpdate(this, elapsed, this:GetParent()) end)
	mtescrollchild:SetScript("OnTextChanged",
		function(this)
			if string.len(this:GetText()) == 0 then MacroToolkitShorten:Enable()
			else MacroToolkitShorten:Disable() end
		end)
	mtescrollchild:SetFontObject("GameFontHighlightSmall")
	mterrorscroll:SetScrollChild(mtescrollchild)

	local mterrort = mterrorscroll:CreateFontString("MacroToolkitErrorLabel", "ARTWORK", "GameFontHighlightSmall")
	mterrort:SetText(_G.ERRORS .. ":")
	mterrort:SetPoint("BOTTOMLEFT", mterrorscroll, "TOPLEFT", -3, 5)
	
	local mttextbutton = CreateFrame("Button", nil, mtframe)
	mttextbutton:SetSize(286, 85)
	mttextbutton:SetPoint("TOPLEFT", mtmfscrollchild)
	mttextbutton:SetScript("OnClick", function() mtmscrollchild:SetFocus() end)

	local mtcancelbutton = CreateFrame("Button", "MacroToolkitCancel", mtframe, "UIPanelButtonTemplate")
	mtcancelbutton:SetText(_G.CANCEL)
	mtcancelbutton:SetFrameStrata("HIGH")
	mtcancelbutton:SetSize(80, 22)
	mtcancelbutton:SetPoint("BOTTOMRIGHT", mtmscroll, "TOPRIGHT", 25, 10)
	mtcancelbutton:SetScript("OnClick", 
		function()
			PlaySound("igMainMenuOptionCheckBoxOn")
			MT:MacroFrameUpdate()
			MacroToolkitPopup:Hide()
			MacroToolkitText:ClearFocus()
		end)

	local mtsavebutton = CreateFrame("Button", "MacroToolkitSave", mtframe, "UIPanelButtonTemplate")
	mtsavebutton:SetText(_G.SAVE)
	mtsavebutton:SetFrameStrata("HIGH")
	mtsavebutton:SetSize(80, 22)
	mtsavebutton:SetPoint("BOTTOM", mtcancelbutton, "TOP", 0, 15)
	mtsavebutton:SetScript("OnClick", 
		function()
			PlaySound("igMainMenuOptionCheckBoxOn")
			MT:SaveMacro()
			MT:MacroFrameUpdate()
			MacroToolkitPopup:Hide()
			MacroToolkitText:ClearFocus()
		end)
	
	local mttextbg = CreateFrame("Frame", "MacroToolkitTextBg", mtframe)
	mttextbg:SetSize(322, 95)
	mttextbg:SetPoint("TOPLEFT", 6, -289)
	mttextbg:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16, tileSize = 16, tile = true, insets = {left = 5, right = 5, top = 5, bottom = 5}})
	mttextbg:SetBackdropBorderColor(_G.TOOLTIP_DEFAULT_COLOR.r, _G.TOOLTIP_DEFAULT_COLOR.g, _G.TOOLTIP_DEFAULT_COLOR.b)
	mttextbg:SetBackdropColor(_G.TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, _G.TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, _G.TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
	
	local mttab1 = CreateFrame("Button", "MacroToolkitFrameTab1", mtframe, "TabButtonTemplate")
	mttab1:SetText(_G.GENERAL_MACROS)
	mttab1:SetID(1)
	mttab1:SetPoint("TOPLEFT", 51, -28)
	PanelTemplates_TabResize(mttab1, -15)
	MacroToolkitFrameTab1HighlightTexture:SetWidth(mttab1:GetTextWidth() + 31)
	mttab1:SetScript("OnClick",
		function(this)
			PlaySound("igMainMenuOptionCheckBoxOn")
			PanelTemplates_SetTab(mtframe, this:GetID())
			MT:SaveMacro()
			MT:SetMacros(true)
			mtmscrollchild:SetText("")
			mtmfscrollchild:SetText("")
			mtescrollchild:SetText("")
			MT:MacroFrameUpdate()
			mtscroll:SetVerticalScroll(0)
			MT:GetLastBackupDate()
		end)
	
	local mttab2 = CreateFrame("Button", "MacroToolkitFrameTab2", mtframe, "TabButtonTemplate")
	mttab2:SetText(_G.GENERAL_MACROS)
	mttab2:SetID(2)
	mttab2:SetPoint("LEFT", mttab1, "RIGHT", 0, 0)
	mttab2:SetFormattedText(_G.CHARACTER_SPECIFIC_MACROS, UnitName("player"))
	PanelTemplates_TabResize(mttab2, -15, nil, nil, 150)
	MacroToolkitFrameTab2HighlightTexture:SetWidth(mttab2:GetTextWidth() + 31)
	mttab2:SetScript("OnEnter",
		function(this)
			if this:GetFontString():IsTruncated() then
				GameTooltip:SetOwner(this, "ANCHOR_RIGHT")
				GameTooltip:SetText(string.format(_G.CHARACTER_SPECIFIC_MACROS, UnitName("player")), 1, 1, 1)
			end
		end)
	mttab2:SetScript("OnLeave", GameTooltip_Hide)
	mttab2:SetScript("OnClick",
		function(this)
			PlaySound("igMainMenuOptionCheckBoxOn")
			PanelTemplates_SetTab(mtframe, this:GetID())
			MT:SaveMacro()
			MT:SetMacros(false)
			mtmscrollchild:SetText("")
			mtmfscrollchild:SetText("")
			mtescrollchild:SetText("")
			MT:MacroFrameUpdate()
			mtscroll:SetVerticalScroll(0)
			MT:GetLastBackupDate()
		end)
	
	local mtdel = CreateFrame("Button", "MacroToolkitDelete", mtframe, "UIPanelButtonTemplate")
	mtdel:SetText(_G.DELETE)
	mtdel:SetSize(80, 22)
	mtdel:SetPoint("BOTTOMLEFT", 4, 4)
	mtdel:SetScript("OnClick",
		function()
			PlaySound("igMainMenuOptionCheckBoxOn")
			local selectedMacro = mtframe.selectedMacro
			if mtmscrollchild.extended then
				local mindex = MT:GetCurrentIndex()
				if mindex then MT:DeleteExtended(mindex) end
			end
			DeleteMacro(selectedMacro)
			local numMacros = select(PanelTemplates_GetSelectedTab(mtframe), GetNumMacros())
			if selectedMacro > (numMacros + mtframe.macroBase) then selectedMacro = selectedMacro - 1 end
			if selectedMacro <= mtframe.macroBase then mtframe.selectedMacro = nil
			else mtframe.selectedMacro = selectedMacro end
			MT:MacroFrameUpdate()
			mtmscrollchild:ClearFocus()
		end)

	local mtex = CreateFrame("Button", "MacroToolkitExtend", mtframe, "UIPanelButtonTemplate")
	mtex:SetText(L["Extend"])
	mtex:SetSize(80, 22)
	mtex:SetPoint("LEFT", mtdel, "RIGHT", 80, 0)
	mtex:SetScript("OnClick", function(this) MT:ExtendClick(this) end)

	local mtshorten = CreateFrame("Button", "MacroToolkitShorten", mtframe, "UIPanelButtonTemplate")
	mtshorten:SetText(L["Shorten"])
	mtshorten:SetSize(80, 22)
	mtshorten:SetPoint("LEFT", mtex, "RIGHT", 00, 0)
	mtshorten:SetScript("OnClick", 
		function()
			local mt, c = MT:ShortenMacro(mtmscrollchild:GetText())
			mtmscrollchild:SetText(mt)
			local m, e = MT:FormatMacro(mt)
			mtmfscrollchild:SetText(m)
			MT:ShowShortened(c)
		end)
		
	local mtnew = CreateFrame("Button", "MacroToolkitNew", mtframe, "UIPanelButtonTemplate")
	mtnew:SetText(_G.NEW)
	mtnew:SetSize(80, 22)
	mtnew:SetPoint("BOTTOMRIGHT", -84, 4)
	mtnew:SetScript("OnClick", 
		function()
			MT:SaveMacro()
			MacroToolkitPopup.mode = "new"
			MacroToolkitPopup:Show()
	end)

	local mtexit = CreateFrame("Button", "MacroToolkitExit", mtframe, "UIPanelButtonTemplate")
	mtexit:SetText(_G.EXIT)
	mtexit:SetSize(80, 22)
	mtexit:SetPoint("BOTTOMRIGHT", -5, 4)
	mtexit:SetScript("OnClick", HideParentPanel)
	
	PanelTemplates_SetNumTabs(mtframe, 2);
	PanelTemplates_SetTab(mtframe, 1);
	mtframe:SetScript("OnShow", 
		function()
			mtframe:SetPoint("BOTTOMLEFT", MT.db.profile.x, MT.db.profile.y)
			MT:MacroFrameUpdate()
			PlaySound("igCharacterInfoOpen")
			UpdateMicroButtons()
			MT:GetLastBackupDate()
			if #MT.db.global.custom == 0 then mtcustomflyout:Disable()
			else mtcustomflyout:Enable() end
		end)
	mtframe:SetScript("OnHide", 
		function()
			MacroToolkitPopup:Hide()
			MT:SaveMacro()
			PlaySound("igCharacterInfoClose")
			UpdateMicroButtons()
		end)

	local ddvalues = {}
	for k, v in ipairs(MT.scripts) do ddvalues[k] = v[1] end
	table.sort(ddvalues, function(a,b) return a < b end)
	local mtaddscript = CreateFrame("Button", "MacroToolkitAddScript", mtframe, "UIPanelButtonTemplate")
	mtaddscript:SetText(_G.KEY_INSERT)
	mtaddscript:SetSize(80, 22)
	mtaddscript:SetPoint("TOPRIGHT", mtframe, "TOPRIGHT", -15, -80)
	mtaddscript:Disable()
	mtaddscript:SetScript("OnClick",
		function(this)
			if not MT.insertspecial then return end
			local script = MT:FindScript(ddvalues[MT.insertspecial])
			local s = MT.slash .. script[2]
			if script[3] == "raidtarget" then s = MT.slash .. string.format(script[2] ..  " %s %s", MT.markertarget or "target", MT.targetmarker or 1)
			elseif script[3] == "buff" then
				local chan = string.lower(MT.channel or "raid")
				s = MT.slash .. script[2] .. ((chan == "raid") and "" or " " .. chan) end
			local l = string.len(s)
			local ls = string.len(mtmscrollchild:GetText())
			if (ls + l) > 255 then
				StaticPopupDialogs["MACROTOOLKIT_TOOLONG"].text = string.format(L["Not enough space. Command requires %d characters (%d available)"], l, 255 - ls)
				StaticPopup_Show("MACROTOOLKIT_TOOLONG")
			else
				if ls + l + 1 < 256 then s = s .. "\n" end
				mtmscrollchild:SetFocus()
				mtmscrollchild:Insert(s)
				local m, e = MT:FormatMacro(mtmscrollchild:GetText())
				mtmfscrollchild:SetText(m)
			end
		end)

	local mtseltarget, mtmarkerdd, mtchan
	local mtscriptdd = AceGUI:Create("Dropdown")
	mtscriptdd.frame:SetParent(mtframe)
	mtscriptdd.label:SetTextColor(1, 1, 1, 1)
	mtscriptdd:SetList(ddvalues)
	mtscriptdd:SetWidth(200)
	mtscriptdd:SetLabel(L["Insert special"] .. ":")
	mtscriptdd:SetCallback("OnValueChanged", 
		function(info, name, key)
			MT.insertspecial = key
			if MT:FindScript(ddvalues[key])[3] == "raidtarget" then
				mtseltarget.frame:Show()
				mtmarkerdd.frame:Show()
				mtseltarget.label:SetTextColor(1, 1, 1, 1)
				mtmarkerdd.label:SetTextColor(1, 1, 1, 1)
			else
				mtseltarget.frame:Hide()
				mtmarkerdd.frame:Hide()
			end
			if MT:FindScript(ddvalues[key])[3] == "buff" then
				mtchan.frame:Show()
				mtchan.label:SetTextColor(1, 1, 1, 1)
			else mtchan.frame:Hide() end
			mtaddscript:Enable()
		end)
	mtscriptdd:SetPoint("TOPLEFT", mtscrolltop, "TOPRIGHT", 0, 0)

	local mtcon = CreateFrame("Button", "MacroToolkitConditions", mtframe, "UIPanelButtonTemplate")
	mtcon:SetSize(80, 22)
	mtcon:SetText(L["Conditions"])
	mtcon:SetPoint("BOTTOMLEFT", mtscriptdd.frame, "TOPLEFT", 0, 7)
	mtcon:SetScript("OnClick",
		function()
			MacroToolkitBuilderFrame:Show()
			mtframe:Hide()
		end)

	local mtchannels = {[_G.CHAT] =_G.CHAT, [_G.PARTY] =_G.PARTY, [_G.RAID] = _G.RAID}
	mtchan = AceGUI:Create("Dropdown")
	mtchan.frame:SetParent(mtframe)
	mtchan.label:SetTextColor(1, 1, 1, 1)
	mtchan:SetList(mtchannels)
	mtchan:SetWidth(100)
	mtchan:SetLabel(L["Report to"] .. ":")
	mtchan:SetCallback("OnValueChanged", function(info, name, key) MT.channel = key end)
	mtchan:SetPoint("TOPLEFT", mtscriptdd.frame, "BOTTOMLEFT", 0, -7)
	mtchan.frame:Hide()
	
	mtseltarget = AceGUI:Create("EditBox")
	mtseltarget.frame:SetParent(mtframe)
	mtseltarget.label:SetTextColor(1, 1, 1, 1)
	mtseltarget:SetWidth(100)
	mtseltarget:SetLabel(_G.TARGET .. ":")
	mtseltarget:SetCallback("OnTextChanged", function(info, name, text) MT.markertarget = text end)
	mtseltarget:SetPoint("TOPLEFT", mtscriptdd.frame, "BOTTOMLEFT", 0, -7)
	mtseltarget.frame:Hide()
	
	local mtmarkers = {}
	for m = 1, 8 do mtmarkers[m] = _G.ICON_LIST[_G.ICON_TAG_LIST["rt" .. m]] .. "0|t" end
	mtmarkerdd = AceGUI:Create("Dropdown")
	mtmarkerdd.frame:SetParent(mtframe)
	mtmarkerdd.label:SetTextColor(1, 1, 1, 1)
	mtmarkerdd:SetList(mtmarkers)
	mtmarkerdd:SetWidth(100)
	mtmarkerdd:SetLabel(L["Raid marker"] .. ":")
	mtmarkerdd:SetCallback("OnValueChanged", function(info, name, key) MT.targetmarker = key end)
	mtmarkerdd:SetPoint("LEFT", mtseltarget.frame, "RIGHT", 20, 0)
	mtmarkerdd.frame:Hide()
	
	local mtaddslot = CreateFrame("Button", "MacroToolkitAddSlot", mtframe, "UIPanelButtonTemplate")
	mtaddslot:SetText(_G.KEY_INSERT)
	mtaddslot:SetSize(80, 22)
	mtaddslot:SetPoint("TOPRIGHT", mtframe, "BOTTOMRIGHT", -15, 240)
	mtaddslot:Disable()
	mtaddslot:SetScript("OnClick",
		function(this)
			if not MT.insertslot then return end
			local slot = tostring(MT.insertslot)
			local l = string.len(slot)
			local ls = string.len(mtmscrollchild:GetText())
			if ls + l > 255 then
				MT.specialchars = l
				StaticPopup_Show("MACROTOOLKIT_TOOLONG")
			else
				mtmscrollchild:SetFocus()
				mtmscrollchild:Insert(slot)
				local m, e = MT:FormatMacro(mtmscrollchild:GetText())
				mtmfscrollchild:SetText(m)
			end
		end)
		
	local mtslotdd = AceGUI:Create("Dropdown")
	mtslotdd.frame:SetParent(mtframe)
	mtslotdd.label:SetTextColor(1, 1, 1, 1)
	mtslotdd:SetList(MT.slots)
	mtslotdd:SetWidth(200)
	mtslotdd:SetLabel(L["Insert slot"] .. ":")
	mtslotdd:SetCallback("OnValueChanged", function(info, name, key) MT.insertslot = key; mtaddslot:Enable() end)
	mtslotdd:SetPoint("BOTTOMLEFT", mtscrollbot, "BOTTOMRIGHT", 0, 5)

	local mtbackup = CreateFrame("Button", "MacroToolkitBackup", mtframe, "UIPanelButtonTemplate")
	mtbackup:SetText(L["Backup"])
	mtbackup:SetSize(80, 22)
	mtbackup:SetPoint("TOPLEFT", mtsavebutton, "TOPRIGHT", 20, 0)
	mtbackup:SetScript("OnClick", function() MT:GetBackupName() end)

	local mtbl = mtframe:CreateFontString("MacroToolkitBL", "ARTWORK", "GameFontNormalSmall")
	mtbl:SetPoint("TOPLEFT", mtbackup, "BOTTOMLEFT", 2, -12)
	mtbl:SetTextColor(0.6, 0.6, 0.6, 1)

	local mtrestore = CreateFrame("Button", "MacroToolkitRestore", mtframe, "UIPanelButtonTemplate")
	mtrestore:SetText(L["Restore"])
	mtrestore:SetSize(80, 22)
	mtrestore:SetPoint("TOPLEFT", mtbackup, "TOPRIGHT", 18, 0)
	mtrestore:SetScript("OnClick", function() MacroToolkitRestoreFrame:Show() end)
	
	local mtclear = CreateFrame("Button", "MacroToolkitClear", mtframe, "UIPanelButtonTemplate")
	mtclear:SetText(_G.CLEAR_ALL)
	mtclear:SetSize(80, 22)
	mtclear:SetPoint("TOPLEFT", mtrestore, "TOPRIGHT", 18, 0)
	mtclear:SetScript("OnClick",
		function()
			local tab = PanelTemplates_GetSelectedTab(MacroToolkitFrame)
			local mtype = _G["MacroToolkitFrameTab" .. tab]:GetText()
			StaticPopupDialogs.MACROTOOLKIT_DELETEBACKUP.text = "|cffeedd82" .. mtype .. "|r\n\n|cffff0000" .. L["Delete all macros"] .. "|r\n\n" .. L["Are you sure? This operation cannot be undone."]
			StaticPopupDialogs.MACROTOOLKIT_DELETEBACKUP.OnAccept = function() MT:ClearAllMacros() end
			StaticPopup_Show("MACROTOOLKIT_DELETEBACKUP")
		end)

	local mtshare = CreateFrame("Button", "MacroToolkitShare", mtframe, "UIPanelButtonTemplate")
	mtshare:SetText(_G.SHARE_QUEST_ABBREV)
	mtshare:SetSize(80, 22)
	mtshare:SetPoint("TOPRIGHT", mtclear, "BOTTOMRIGHT", 0, -15)
	mtshare:SetScript("OnClick",
		function()
			if not mtframe.selectedMacro then return end
			local name = GetMacroInfo(mtframe.selectedMacro)
			StaticPopupDialogs.MACROTOOLKIT_SHARE.text = "|cffeedd82" .. name .. "|r\n\n" .. _G.SHARE_QUEST_ABBREV
			StaticPopup_Show("MACROTOOLKIT_SHARE")
		end)
	return mtframe
end

function MT:CreateRestoreFrame()
	local mtrf = CreateFrame("Frame", "MacroToolkitRestoreFrame", UIParent)
	mtrf:SetMovable(true)
	mtrf:EnableMouse(true)
	mtrf:SetSize(297, 298)
	mtrf:SetPoint("TOPLEFT", MacroToolkitFrame, "TOPRIGHT", 0, 0)
	mtrf:Hide()
	
	local mtrftl = mtrf:CreateTexture("BACKGROUND")
	mtrftl:SetTexture("Interface\\MacroFrame\\MacroPopup-TopLeft")
	mtrftl:SetTexCoord(0, 1, 0.234375, 1)
	mtrftl:SetSize(256, 256)
	mtrftl:SetPoint("TOPLEFT")

	local mtrftr = mtrf:CreateTexture("BACKGROUND")
	mtrftr:SetTexture("Interface\\MacroFrame\\MacroPopup-TopRight")
	mtrftr:SetTexCoord(0, 1, 0.234375, 1)
	mtrftr:SetSize(64, 256)
	mtrftr:SetPoint("TOPLEFT", 256, 0)
	
	local mtrfbl = mtrf:CreateTexture("BACKGROUND")
	mtrfbl:SetTexture("Interface\\MacroFrame\\MacroPopup-BotLeft")
	mtrfbl:SetSize(256, 64)
	mtrfbl:SetPoint("TOPLEFT", 0, -256)
	
	local mtrfbr = mtrf:CreateTexture("BACKGROUND")
	mtrfbr:SetTexture("Interface\\MacroFrame\\MacroPopup-BotRight")
	mtrfbr:SetSize(64, 64)
	mtrfbr:SetPoint("TOPLEFT", 256, -256)
	
	local mtrftitle = mtrf:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	mtrftitle:SetText(L["Manage Backups"])
	mtrftitle:SetPoint("CENTER", 0, 120)

	local mtrftype = mtrf:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	mtrftype:SetPoint("CENTER", 0, 105)
	mtrftype:SetTextColor(0.5, 0.5, 0.5, 1)

	local mtrfdesc = mtrf:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
	mtrfdesc:SetPoint("CENTER", 0, -20)

	local mtrfdd = AceGUI:Create("Dropdown")
	mtrfdd.frame:SetParent(mtrf)
	mtrfdd.label:SetTextColor(1, 1, 1, 0.75)
	mtrfdd:SetWidth(200)
	mtrfdd:SetLabel(L["Backup"] .. ":")
	mtrfdd:SetCallback("OnValueChanged",
		function(info, name, key)
			MT.backup = key
			mtrfdesc:SetText(MT:GetBackupTitle(key))
			MacroToolkitRestoreRestore:Enable()
			MacroToolkitRestoreDelete:Enable()
		end)
	mtrfdd:SetPoint("CENTER", 0, 60)
	
	local mtrfrestore = CreateFrame("Button", "MacroToolkitRestoreRestore", mtrf, "UIPanelButtonTemplate")
	mtrfrestore:SetText(L["Restore"])
	mtrfrestore:SetSize(115, 22)
	mtrfrestore:SetPoint("BOTTOMLEFT", 11, 13)
	mtrfrestore:Disable()
	mtrfrestore:SetScript("OnClick",
		function()
			local tab = PanelTemplates_GetSelectedTab(MacroToolkitFrame)
			local mtype = _G["MacroToolkitFrameTab" .. tab]:GetText()
			StaticPopupDialogs.MACROTOOLKIT_DELETEBACKUP.text = "|cffeedd82" .. mtype .. "|r\n\n|cffff0000" .. L["Restore Backup"] .. "|r\n\n" .. L["Are you sure? This operation cannot be undone."]
			StaticPopupDialogs.MACROTOOLKIT_DELETEBACKUP.OnAccept = function() MT:RestoreBackup() end
			StaticPopup_Show("MACROTOOLKIT_DELETEBACKUP")
		end)
		
	local mtrfcancel = CreateFrame("Button", "MacroToolkitRestoreCancel", mtrf, "UIPanelButtonTemplate")
	mtrfcancel:SetText(_G.CANCEL)
	mtrfcancel:SetSize(78, 22)
	mtrfcancel:SetPoint("BOTTOMRIGHT", -11, 13)
	mtrfcancel:SetScript("OnClick",
		function()
			PlaySound("gsTitleOptionOK")
			mtrf:Hide()
			MT:MacroFrameUpdate()
		end)

	local mtrfdelete = CreateFrame("Button", "MacroToolkitRestoreDelete", mtrf, "UIPanelButtonTemplate")
	mtrfdelete:SetText(_G.DELETE)
	mtrfdelete:SetSize(78, 22)
	mtrfdelete:SetPoint("RIGHT", mtrfcancel, "LEFT", -2, 0)
	mtrfdelete:Disable()
	mtrfdelete:SetScript("OnClick",
		function()
			local tab = PanelTemplates_GetSelectedTab(MacroToolkitFrame)
			local mtype = _G["MacroToolkitFrameTab" .. tab]:GetText()
			StaticPopupDialogs.MACROTOOLKIT_DELETEBACKUP.text = "|cffeedd82" .. mtype .. "|r\n\n|cffff0000" .. L["Delete Backup"] .. "|r\n\n" .. L["Are you sure? This operation cannot be undone."]
			StaticPopupDialogs.MACROTOOLKIT_DELETEBACKUP.OnAccept = function() MT:DeleteBackup(mtrfdd, mtrfdesc) end
			StaticPopup_Show("MACROTOOLKIT_DELETEBACKUP")
		end)

	mtrf:SetScript("OnShow", 
		function(this)
			PlaySound("igCharacterInfoOpen")
			local tab = PanelTemplates_GetSelectedTab(MacroToolkitFrame)
			mtrftype:SetText(_G["MacroToolkitFrameTab" .. tab]:GetText())
			MacroToolkitEdit:Disable()
			MacroToolkitDelete:Disable()
			MacroToolkitNew:Disable()
			MacroToolkitFrameTab1:Disable()
			MacroToolkitFrameTab2:Disable()
			MacroToolkitShorten:Disable()
			MacroToolkitExtend:Disable()
			MacroToolkitBackup:Disable()
			MacroToolkitRestore:Disable()
			MacroToolkitClear:Disable()
			MacroToolkitShare:Disable()
			MacroToolkitSave:Disable()
			MacroToolkitCancel:Disable()
			MacroToolkitFlyout:Disable()
			MacroToolkitCustom:Disable()
			MacroToolkitConditions:Disable()
			mtrfdd:SetList(MT:GetBackups())
			if MT.backup then
				mtrfrestore:Enable()
				mtrfdelete:Enable()
			end
		end)

	mtrf:SetScript("OnHide", 
		function(this)
			MacroToolkitEdit:Enable()
			MacroToolkitDelete:Enable()
			MacroToolkitShorten:Enable()
			MacroToolkitExtend:Enable()
			MacroToolkitBackup:Enable()
			MacroToolkitClear:Enable()
			MacroToolkitShare:Enable()
			MacroToolkitFrameTab1:Enable()
			MacroToolkitFrameTab2:Enable()
			MacroToolkitSave:Enable()
			MacroToolkitCancel:Enable()
			MacroToolkitConditions:Enable()
			MT:GetLastBackupDate()
			if #MT.db.global.custom > 0 then MacroToolkitFlyout:Enable() end
			MacroToolkitCustom:Enable()
		end)
end

function MT:CreateMTPopup()
	local mtpf = CreateFrame("Frame", "MacroToolkitPopup", UIParent)
	mtpf:SetMovable(true)
	mtpf:EnableMouse(true)
	mtpf:SetSize(297, 411)
	mtpf:SetPoint("TOPLEFT", MacroToolkitFrame, "TOPRIGHT", 0, 0)
	mtpf:Hide()

	local mtpftl = mtpf:CreateTexture("BACKGROUND")
	mtpftl:SetTexture("Interface\\MacroFrame\\MacroPopup-TopLeft")
	mtpftl:SetSize(256, 368)
	mtpftl:SetPoint("TOPLEFT")

	local mtpftr = mtpf:CreateTexture("BACKGROUND")
	mtpftr:SetTexture("Interface\\MacroFrame\\MacroPopup-TopRight")
	mtpftr:SetSize(64, 368)
	mtpftr:SetPoint("TOPLEFT", 256, 0)
	
	local mtpfbl = mtpf:CreateTexture("BACKGROUND")
	mtpfbl:SetTexture("Interface\\MacroFrame\\MacroPopup-BotLeft")
	mtpfbl:SetSize(256, 64)
	mtpfbl:SetPoint("TOPLEFT", 0, -368)
	
	local mtpfbr = mtpf:CreateTexture("BACKGROUND")
	mtpfbr:SetTexture("Interface\\MacroFrame\\MacroPopup-BotRight")
	mtpfbr:SetSize(64, 64)
	mtpfbr:SetPoint("TOPLEFT", 256, -368)
	
	local mtpftitle = mtpf:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
	mtpftitle:SetText(_G.MACRO_POPUP_TEXT)
	mtpftitle:SetPoint("TOPLEFT", 24, -21)

	local mtpfcicon = mtpf:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
	mtpfcicon:SetText(_G.MACRO_POPUP_CHOOSE_ICON)
	mtpfcicon:SetPoint("TOPLEFT", 24, -69)
	
	local mtpfedit = CreateFrame("EditBox", "MacroToolkitPopupEdit", mtpf)
	mtpfedit:SetMaxLetters(16)
	mtpfedit:SetSize(182, 20)
	mtpfedit:SetPoint("TOPLEFT", 29, -35)
	mtpfedit:SetFontObject("ChatFontNormal")
	
	local mtpfeditl = mtpfedit:CreateTexture("MacroToolkitFramePopupL", "BACKGROUND")
	mtpfeditl:SetTexture("Interface\\ClassTrainerFrame\\UI-ClassTrainer-FilterBorder")
	mtpfeditl:SetSize(12, 29)
	mtpfeditl:SetPoint("TOPLEFT", -11, 0)
	mtpfeditl:SetTexCoord(0, 0.09375, 0, 1)

	local mtpfeditm = mtpfedit:CreateTexture("MacroToolkitFramePopupM", "BACKGROUND")
	mtpfeditm:SetTexture("Interface\\ClassTrainerFrame\\UI-ClassTrainer-FilterBorder")
	mtpfeditm:SetSize(175, 29)
	mtpfeditm:SetPoint("LEFT", mtpfeditl, "RIGHT")
	mtpfeditm:SetTexCoord(0.09375, 0.90625, 0, 1)

	local mtpfeditr = mtpfedit:CreateTexture("MacroToolkitFramePopupR", "BACKGROUND")
	mtpfeditr:SetTexture("Interface\\ClassTrainerFrame\\UI-ClassTrainer-FilterBorder")
	mtpfeditr:SetSize(12, 29)
	mtpfeditr:SetPoint("LEFT", mtpfeditm, "RIGHT")
	mtpfeditr:SetTexCoord(0.90625, 1, 0, 1)

	mtpfedit:SetScript("OnTextChanged",
		function(this)
			local text = string.gsub(this:GetText(), "\"", "")
			MT:PopupOkayUpdate()
			MacroToolkitSelMacroName:SetText(text)
		end)
	
	mtpfedit:SetScript("OnEscapePressed", function() MT:CancelEdit() end)
	mtpfedit:SetScript("OnEnterPressed", function() if MacroToolkitPopupOk:IsEnabled() then MT:PopupOkayButtonOnClick(MacroToolkitPopupOk) end end)

	local mtpfscroll = CreateFrame("ScrollFrame", "MacroToolkitPopupScroll", mtpf, "FauxScrollFrameTemplate")
	mtpfscroll:SetSize(296, 276)
	mtpfscroll:SetPoint("TOPRIGHT", -39, -98)
	mtpfscroll:SetScript("OnVerticalScroll", function(this, offset) FauxScrollFrame_OnVerticalScroll(this, offset, _G.MACRO_ICON_ROW_HEIGHT, function() MT:PopupUpdate(this) end) end)

	local sb = MacroToolkitPopupScrollScrollBar
	sb:ClearAllPoints()
	sb:SetPoint("TOPLEFT", mtpfscroll, "TOPRIGHT", 6, -22)
	sb:SetPoint("BOTTOMLEFT", mtpfscroll, "BOTTOMRIGHT", 6, 22)
	
	MacroToolkitPopupScrollScrollBarScrollUpButton:ClearAllPoints()
	MacroToolkitPopupScrollScrollBarScrollUpButton:SetPoint("BOTTOM", sb, "TOP", 0, 5)
	MacroToolkitPopupScrollScrollBarScrollDownButton:ClearAllPoints()
	MacroToolkitPopupScrollScrollBarScrollDownButton:SetPoint("TOP", sb, "BOTTOM", 0, -5)
	
	local sbt = mtpfscroll:CreateTexture("MacroToolkitFramePopupSBT", "BACKGROUND")
	sbt:SetTexture("Interface\\ClassTrainerFrame\\UI-ClassTrainer-ScrollBar")
	sbt:SetSize(30, 140)
	sbt:SetPoint("TOPLEFT", mtpfscroll, "TOPRIGHT", -3, 2)
	sbt:SetTexCoord(0, 0.46875, 0.0234375, 0.9609375)

	local sbb = mtpfscroll:CreateTexture("MacroToolkitFramePopupSBB", "BACKGROUND")
	sbb:SetTexture("Interface\\ClassTrainerFrame\\UI-ClassTrainer-ScrollBar")
	sbb:SetSize(30, 140)
	sbb:SetPoint("BOTTOMLEFT", mtpfscroll, "BOTTOMRIGHT", -3, -2)
	sbb:SetTexCoord(0.53125, 1, 0.03125, 1)

	for mpb = 1, 30 do
		local b = CreateFrame("CheckButton", "MacroToolkitPopupButton" .. mpb, mtpf, "MacroPopupButtonTemplate")
		if mpb == 1 then b:SetPoint("TOPLEFT", 24, -105)
		elseif mpb == 6 or mpb == 11 or mpb == 16 or mpb == 21 or mpb == 26 then b:SetPoint("TOPLEFT", _G["MacroToolkitPopupButton" .. (mpb - 5)], "BOTTOMLEFT", 0, -8)
		else b:SetPoint("LEFT", _G["MacroToolkitPopupButton" .. (mpb - 1)], "RIGHT", 10, 0) end
		b:SetID(mpb)
		b:SetScript("OnClick", function(this) MT:PopupButtonOnClick(this) end)
	end

	local mtpfcancel = CreateFrame("Button", "MacroToolkitPopupCancel", mtpf, "UIPanelButtonTemplate")
	mtpfcancel:SetText(_G.CANCEL)
	mtpfcancel:SetSize(78, 22)
	mtpfcancel:SetPoint("BOTTOMRIGHT", -11, 13)
	mtpfcancel:SetScript("OnClick",
		function()
			MT:CancelEdit()
			PlaySound("gsTitleOptionOK")
		end)

	local mtpfok = CreateFrame("Button", "MacroToolkitPopupOk", mtpf, "UIPanelButtonTemplate")
	mtpfok:SetText(_G.OKAY)
	mtpfok:SetSize(78, 22)
	mtpfok:SetPoint("RIGHT", mtpfcancel, "LEFT", -2, 0)
	mtpfok:SetScript("OnClick",
		function()
			local index = 1
			local iconTexture = MT:GetSpellorMacroIconInfo(MacroToolkitPopup.selectedIcon)
			local text = MacroToolkitPopupEdit:GetText()
			text = string.gsub(text, "\"", "")
			if MacroToolkitPopup.mode == "new" then
				index = CreateMacro(text, iconTexture, nil, (MacroToolkitFrame.macroBase > 0))
				MacroToolkitText.extended = nil
			elseif MacroToolkitPopup.mode == "edit" then index = EditMacro(MacroToolkitFrame.selectedMacro, text, iconTexture) end
			MacroToolkitPopup:Hide()
			MacroToolkitFrame.selectedMacro = index
			MT:MacroFrameUpdate()
			PlaySound("gsTitleOptionOK")
		end)

	mtpf:SetScript("OnShow", 
		function(this)
			mtpfedit:SetFocus()
			PlaySound("igCharacterInfoOpen")
			MT:RefreshPlayerSpellIconInfo()
			MT:PopupUpdate(mtpf)
			MT:PopupOkayUpdate()
			if this.mode == "new" then
				MacroToolkitText:Hide()
				MacroToolkitFauxText:Hide()
				MT:SelectTexture(1)
			end
			MacroToolkitEdit:Disable()
			MacroToolkitDelete:Disable()
			MacroToolkitNew:Disable()
			MacroToolkitFrameTab1:Disable()
			MacroToolkitFrameTab2:Disable()
			MacroToolkitShorten:Disable()
			MacroToolkitExtend:Disable()
			MacroToolkitBackup:Disable()
			MacroToolkitRestore:Disable()
			MacroToolkitClear:Disable()
			MacroToolkitShare:Disable()
			MacroToolkitFlyout:Disable()
			MacroToolkitCustom:Disable()
			MacroToolkitConditions:Disable()
		end)

	mtpf:SetScript("OnHide", 
		function(this)
			if this.mode == "new" then
				MacroToolkitText:Show()
				MacroToolkitFauxText:Show()
				MacroToolkitText:SetFocus()
			end
			MacroToolkitEdit:Enable()
			MacroToolkitDelete:Enable()
			MacroToolkitShorten:Enable()
			MacroToolkitExtend:Enable()
			MacroToolkitBackup:Enable()
			MacroToolkitRestore:Enable()
			MacroToolkitClear:Enable()
			MacroToolkitShare:Enable()
			MacroToolkitConditions:Enable()
			if #MT.db.global.custom > 0 then MacroToolkitFlyout:Enable() end
			MacroToolkitCustom:Enable()
			PlaySound("igCharacterInfoClose")
			local numMacros
			local numAccountMacros, numCharacterMacros = GetNumMacros()
			numMacros = (MacroToolkitFrame.macroBase == 0) and numAccountMacros or numCharacterMacros
			if numMacros < MacroToolkitFrame.macroMax then MacroToolkitNew:Enable() end
			PanelTemplates_UpdateTabs(MacroToolkitFrame)
			_G.MACRO_ICON_FILENAMES = nil
			collectgarbage()
		end)

	return mtpf
end

function MT:CreateScriptFrame()
	local mtsf = CreateFrame("Frame", "MacroToolkitScriptFrame", UIParent, "ButtonFrameTemplate")

	local function frameMouseUp()
		mtsf:StopMovingOrSizing()
		MT.db.profile.x = mtsf:GetLeft()
		MT.db.profile.y = mtsf:GetBottom()
	end

	mtsf:SetMovable(true)
	mtsf:EnableMouse(true)
	mtsf:SetSize(638, 424)
	mtsf:SetMovable(true)
	mtsf:EnableMouse(true)
	mtsf:SetPoint("BOTTOMLEFT", MT.db.profile.x, MT.db.profile.y)
	mtsf:SetScript("OnMouseDown", function() mtsf:StartMoving() end)
	mtsf:SetScript("OnMouseUp", frameMouseUp)
	mtsf:Hide()

	local mtstitle = mtsf:CreateFontString(nil, "BORDER", "GameFontNormal")
	mtstitle:SetText(L["Custom slash command"])
	mtstitle:SetPoint("TOP", 0, -5)
	
	local mtsfportrait = mtsf:CreateTexture(nil, "OVERLAY", nil, -1)
	mtsfportrait:SetTexture("Interface\\LFGFRAME\\BattlenetWorking0")
	mtsfportrait:SetSize(96, 96)
	mtsfportrait:SetPoint("TOPLEFT", -25, 25)
	
	local mtsname = mtsf:CreateFontString("MacroToolkitScriptName", "ARTWORK", "GameFontNormalLarge")
	mtsname:SetSize(0, 0)
	mtsname:SetPoint("CENTER", 0, 175)
	mtsname:SetTextColor(MT:HexToRGB(MT.db.profile.commandcolour))

	local mthint = mtsf:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	mthint:SetText(L["Arguments can be accessed using the variables arg1 to arg4"])
	mthint:SetSize(180, 32)
	mthint:SetTextColor(1, 1, 1, 0.75)
	mthint:SetPoint("TOPRIGHT", -7, -25)

	local mtshbleft = mtsf:CreateTexture(nil, "ARTWORK")
	mtshbleft:SetTexture("Interface\\ClassTrainerFrame\\UI-ClassTrainer-HorizontalBar")
	mtshbleft:SetSize(556, 16)
	mtshbleft:SetPoint("TOPLEFT", 2, -300)
	mtshbleft:SetTexCoord(0, 1, 0, 0.25)

	local mtshbright = mtsf:CreateTexture(nil, "ARTWORK")
	mtshbright:SetTexture("Interface\\ClassTrainerFrame\\UI-ClassTrainer-HorizontalBar")
	mtshbright:SetSize(75, 16)
	mtshbright:SetPoint("LEFT", mtshbleft, "RIGHT", 0, 0)
	mtshbright:SetTexCoord(0, 0.29296875, 0.25, 0.5)

	local mtenters = mtsf:CreateFontString("MacroToolkitScriptEnter", "ARTWORK", "GameFontHighlightSmall")
	mtenters:SetText(L["Enter script"] .. ":")
	mtenters:SetPoint("TOPLEFT", 50, -48)

	local mtsexit = CreateFrame("Button", "MacroToolkitScriptExit", mtsf, "UIPanelButtonTemplate")
	mtsexit:SetText(_G.EXIT)
	mtsexit:SetSize(80, 22)
	mtsexit:SetPoint("BOTTOMRIGHT", -5, 4)
	mtsexit:SetScript("OnClick", HideParentPanel)
	
	local mtsdel = CreateFrame("Button", "MacroToolkitScriptDelete", mtsf, "UIPanelButtonTemplate")
	mtsdel:SetText(_G.DELETE)
	mtsdel:SetSize(80, 22)
	mtsdel:SetPoint("BOTTOMLEFT", 4, 4)
	mtsdel:SetScript("OnClick",
		function()
			local delindex
			for i, s in ipairs(MT.db.global.custom) do
				if s.n == MT.newslash then
					delindex = i
					break
				end
			end
			table.remove(MT.db.global.custom, delindex)
			_G["SLASH_MACROTOOLKIT_CUSTOM_" .. string.upper(MT.newslash)] = nil
			SlashCmdList["MACROTOOLKIT_CUSTOM_" .. string.upper(MT.newslash)] = nil
			HideUIPanel(mtsf)
		end)

	local mtsscrollbg = CreateFrame("Frame", "MacroToolkitScriptScrollBg", mtsf)
	mtsscrollbg:SetSize(623, 244)
	mtsscrollbg:SetPoint("TOPLEFT", 7, -62)
	mtsscrollbg:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16, tileSize = 16, tile = true, insets = {left = 5, right = 5, top = 5, bottom = 5}})
	mtsscrollbg:SetBackdropBorderColor(_G.TOOLTIP_DEFAULT_COLOR.r, _G.TOOLTIP_DEFAULT_COLOR.g, _G.TOOLTIP_DEFAULT_COLOR.b)
	mtsscrollbg:SetBackdropColor(_G.TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, _G.TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, _G.TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
	
	local mtsscroll = CreateFrame("ScrollFrame", "MacroToolkitScriptScroll", mtsf, "UIPanelScrollFrameTemplate")
	mtsscroll:SetSize(589, 230)
	mtsscroll:SetPoint("TOPLEFT", mtsscrollbg, 6, -8)
	
	local mtsscrollchild = CreateFrame("EditBox", "MacroToolkitScriptText", mtsf)
	mtsscrollchild:SetMultiLine(true)
	mtsscrollchild:SetSize(589, 230)
	mtsscrollchild:SetScript("OnCursorChanged", ScrollingEdit_OnCursorChanged)
	mtsscrollchild:SetScript("OnUpdate", function(this, elapsed) ScrollingEdit_OnUpdate(this, elapsed, this:GetParent()) end)
	mtsscrollchild:SetScript("OnEscapePressed", EditBox_ClearFocus)
	mtsscrollchild:SetFontObject("GameFontHighlightSmall")
	mtsscroll:SetScrollChild(mtsscrollchild)
	
	local mtserrorbg = CreateFrame("Frame", "MacroToolkitScriptErrorBg", mtsf)
	mtserrorbg:SetSize(623, 90)
	mtserrorbg:SetPoint("BOTTOMLEFT", 7, 27)
	mtserrorbg:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16, tileSize = 16, tile = true, insets = {left = 5, right = 5, top = 5, bottom = 5}})
	mtserrorbg:SetBackdropBorderColor(_G.TOOLTIP_DEFAULT_COLOR.r, _G.TOOLTIP_DEFAULT_COLOR.g, _G.TOOLTIP_DEFAULT_COLOR.b)
	mtserrorbg:SetBackdropColor(_G.TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, _G.TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, _G.TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
	
	local mtserrorscroll = CreateFrame("ScrollFrame", "MacroToolkitScriptErrors", mtsf, "UIPanelScrollFrameTemplate")
	mtserrorscroll:SetSize(589, 76)
	mtserrorscroll:SetPoint("TOPLEFT", mtserrorbg, 6, -8)
	
	local mtsescrollchild = CreateFrame("EditBox", "MacroToolkitScrollErrorText", mtsf)
	mtsescrollchild:SetMultiLine(true)
	mtsescrollchild:SetSize(589, 76)	
	mtsescrollchild:Disable()
	mtsescrollchild:SetScript("OnUpdate", function(this, elapsed) ScrollingEdit_OnUpdate(this, elapsed, this:GetParent()) end)
	mtsescrollchild:SetFontObject("GameFontHighlightSmall")
	mtserrorscroll:SetScrollChild(mtsescrollchild)
	
	local mtstextbutton = CreateFrame("Button", "MacroToolkitScriptFocus", mtsf)
	mtstextbutton:SetSize(589, 230)
	mtstextbutton:SetPoint("TOPLEFT", mtsscrollchild)
	mtstextbutton:SetScript("OnClick", function() mtsscrollchild:SetFocus() end)
	
	local mtssave = CreateFrame("Button", "MacroToolkitScriptSave", mtsf, "UIPanelButtonTemplate")
	mtssave:SetText(_G.SAVE)
	mtssave:SetSize(80, 22)
	mtssave:SetPoint("RIGHT", mtsexit, "LEFT")
	mtssave:SetScript("OnClick",
		function()
			local luatext = mtsscrollchild:GetText()
			local slashname = mtsname:GetText()
			local func, err = loadstring(luatext, slashname)
			if err then
				PlaySoundFile("Sound/INTERFACE/igQuestFailed.ogg")
				mtsescrollchild:SetText("|c" .. MT.db.profile.errorcolour .. err .. "|r\n\n" .. L["Save failed"])
			else
				table.insert(MT.db.global.custom, {n = MT.newslash, s = luatext})
				_G["SLASH_MACROTOOLKIT_CUSTOM_" .. string.upper(MT.newslash) .. "1"] = slashname
				SlashCmdList["MACROTOOLKIT_CUSTOM_" .. string.upper(MT.newslash)] = function(input) MT:DoCustomCommand(luatext, input) end
				HideUIPanel(mtsf)
			end
		end)
	
	mtsf:SetScript("OnShow",
		function()
			mtsf:SetPoint("BOTTOMLEFT", MT.db.profile.x, MT.db.profile.y)
			mtsname:SetText(MT.newslash and (MT.slash .. MT.newslash) or _G.UNKNOWN)
			mtsescrollchild:SetText("")
			local scripttext = ""
			for _, s in ipairs(MT.db.global.custom) do
				if s.n == MT.newslash then
					scripttext = s.s
					break
				end
			end
			mtsscrollchild:SetText(scripttext)
		end)
	
	mtsf:SetScript("OnHide", function() ShowUIPanel(MacroToolkitFrame) end)
end

function MT:CreateSecureFrames()
	for b = 1, _G.MAX_ACCOUNT_MACROS + _G.MAX_CHARACTER_MACROS do
		local frame = CreateFrame("Button", "MacroToolkitSecureButton" .. b, nil, "SecureActionButtonTemplate")
		frame:SetID(b)
		frame:SetAttribute("type", "macro")
		frame:SetAttribute("macrotext", "")
	end
end

function MT:CreateBuilderFrame()
	local mtbf = CreateFrame("Frame", "MacroToolkitBuilderFrame", UIParent, "ButtonFrameTemplate")

	local function frameMouseUp()
		mtbf:StopMovingOrSizing()
		MT.db.profile.x = mtbf:GetLeft()
		MT.db.profile.y = mtbf:GetBottom()
	end

	mtbf:SetMovable(true)
	mtbf:EnableMouse(true)
	mtbf:SetSize(638, 424)
	mtbf:SetMovable(true)
	mtbf:EnableMouse(true)
	mtbf:SetPoint("BOTTOMLEFT", MT.db.profile.x, MT.db.profile.y)
	mtbf:SetScript("OnMouseDown", function() mtbf:StartMoving() end)
	mtbf:SetScript("OnMouseUp", frameMouseUp)
	mtbf:Hide()

	local mtstitle = mtbf:CreateFontString(nil, "BORDER", "GameFontNormal")
	mtstitle:SetText(L["Condition Builder"])
	mtstitle:SetPoint("TOP", 0, -5)
	
	local mtbfportrait = mtbf:CreateTexture(nil, "OVERLAY", nil, -1)
	mtbfportrait:SetTexture("Interface\\TUTORIALFRAME\\UI-HELP-PORTRAIT")
	mtbfportrait:SetSize(60, 60)
	mtbfportrait:SetPoint("TOPLEFT", -5, 5)

	local ddtargets = {
		["zero"] = "",
		["target"] = L["target"], ["targettarget"] = L["targettarget"],
		["player"] = L["player"], ["playertarget"] = L["playertarget"],
		["playertargettarget"] = L["playertargettarget"],
		["pet"] = L["pet"], ["pettarget"] = L["pettarget"],
		["pettargettarget"] = L["pettargettarget"],
		["focus"] = L["focus"], ["focustarget"] = L["focustarget"],
		["focustargettarget"] = L["focustargettarget"],
		["mouseover"] = L["mouseover"], ["mouseovertarget"] = L["mouseovertarget"],
		["npc"] = L["npc"], ["none"] = L["none"], ["other"] = L["other"]
	}
	local ddtorder = {"zero","player","playertarget","playertargettarget","target",
		"targettarget","pet","pettarget","pettargettarget","focus","focustarget",
		"focustargettarget","mouseover","mouseovertarget","npc","none","other"}

	local mtbfother
	local mtbftarget = AceGUI:Create("Dropdown")
	mtbftarget.frame:SetParent(mtbf)
	mtbftarget.label:SetTextColor(1, 1, 1, 1)
	mtbftarget:SetList(ddtargets, ddtorder)
	mtbftarget:SetWidth(200)
	mtbftarget:SetLabel(_G.TARGET .. ":")
	mtbftarget:SetCallback("OnValueChanged", 
		function(info, name, key)
			if key == "other" then mtbfother.frame:Show()
			else mtbfother.frame:Hide() end
			MT.builder.target = key
		end)
	mtbftarget:SetPoint("TOPLEFT", 15, -65)
	
	mtbfother = AceGUI:Create("EditBox")
	mtbfother.frame:SetParent(mtbf)
	mtbfother.label:SetTextColor(1, 1, 1, 1)
	mtbfother:SetWidth(200)
	mtbfother:SetLabel(L["Enter target"] .. ":")
	mtbfother:SetCallback("OnEnterPressed", function(info, target) MT.builder.othertarget = target end)
	mtbfother:SetPoint("LEFT", mtbftarget.frame, "RIGHT", 10, 0)
	mtbfother.frame:Hide()
	
	local ddtoptions = {
		["zero"] = "",
		["dead"] = L["dead"], ["nodead"] = L["nodead"],
		["exists"] = L["exists"], ["noexists"] = L["noexists"], ["harm"] = L["harm"],
		["help"] = L["help"], ["party"] = L["party"], ["raid"] = L["raid"],
		["unithasvehicleui"] = L["unithasvehicleui"]
	}
	local ddoorder = {"zero","nodead","dead","exists","noexists","harm","help","party","raid","unithasvehicleui"}
	local function tovaluechanged(key, index) MT.builder["targetoption" .. index] = (key == "zero") and "" or key end
	
	local mtbftopts1 = AceGUI:Create("Dropdown")
	mtbftopts1.frame:SetParent(mtbf)
	mtbftopts1:SetList(ddtoptions, ddoorder)
	mtbftopts1:SetWidth(150)
	mtbftopts1:SetLabel(L["Status of target"])
	mtbftopts1.label:SetTextColor(1, 1, 1, 1)
	mtbftopts1:SetCallback("OnValueChanged", function(info, name, key) tovaluechanged(key, 1) end)
	mtbftopts1:SetPoint("TOPLEFT", mtbftarget.frame, "BOTTOMLEFT", 0, -5)
	
	local mtbftopts2 = AceGUI:Create("Dropdown")
	mtbftopts2.frame:SetParent(mtbf)
	mtbftopts2:SetList(ddtoptions, ddoorder)
	mtbftopts2:SetWidth(150)
	mtbftopts2:SetCallback("OnValueChanged", function(info, name, key) tovaluechanged(key, 2) end)
	mtbftopts2:SetPoint("LEFT", mtbftopts1.frame, "RIGHT", 3, -9)
	
	local mtbftopts3 = AceGUI:Create("Dropdown")
	mtbftopts3.frame:SetParent(mtbf)
	mtbftopts3:SetList(ddtoptions, ddoorder)
	mtbftopts3:SetWidth(150)
	mtbftopts3:SetCallback("OnValueChanged", function(info, name, key) tovaluechanged(key, 3) end)
	mtbftopts3:SetPoint("LEFT", mtbftopts2.frame, "RIGHT", 3, 0)
	
	local mtbftopts4 = AceGUI:Create("Dropdown")
	mtbftopts4.frame:SetParent(mtbf)
	mtbftopts4:SetList(ddtoptions, ddoorder)
	mtbftopts4:SetWidth(150)
	mtbftopts4:SetCallback("OnValueChanged", function(info, name, key) tovaluechanged(key, 4) end)
	mtbftopts4:SetPoint("LEFT", mtbftopts3.frame, "RIGHT", 3, 0)
	
	local ddotherorder = {"zero",
		"bar", "bonusbar", "btn", "channeling", "nochanneling", "combat", "nocombat", "cursor",
		"extrabar", "flyable", "noflyable", "flying", "noflying", "form", "indoors",
		"outdoors", "mod", "nomod", "mounted", "nomounted", "overridebar", "party", "raid",
		"pet", "nopet", "petbattle", "possessbar", "spec", "stealth", "nostealth",
		"swimming", "noswimming", "vehicleui", "novehicleui", "worn",
	}
	L["zero"] = ""
	local ddother = {}
	for _, o in ipairs(ddotherorder) do ddother[o] = L[o] end

	local function addvaluechanged(key, row, pdd, eb)
		local realkey = key
		local negate
		local ddvals = {}
		if string.sub(key, 1, 2) == "no" then
			realkey = string.sub(key, 3)
			negate = true
		end
		local ctype = negate and 0 or (MT.conditions[realkey] or 0)
		if ctype == 1 then
			ddvals.zero = ""
			for n = 1, 6 do ddvals[tostring(n)] = n end
			pdd:SetList(ddvals)
			pdd.frame:Show()
			eb.frame:Hide()
		elseif ctype == 2 or ctype == 3 then
			pdd.frame:Hide()
			eb.frame:Show()
		elseif ctype == 4 then
			local b = {"zero","party","raid"}
			for _, bt in ipairs(b) do ddvals[bt] = L[bt] end
			pdd:SetList(ddvals, b)
			pdd.frame:Show()
			eb.frame:Hide()
		elseif ctype == 5 then
			local b = {"zero","alt","shift","ctrl"}
			for _, bt in ipairs(b) do ddvals[bt] = L[bt] end
			pdd:SetList(ddvals, b)
			pdd.frame:Show()
			eb.frame:Hide()
		elseif ctype == 6 then
			local b = {"zero","LeftButton","MiddleButton","RightButton","Button4","Button5"}
			for _, bt in ipairs(b) do ddvals[bt] = L[bt] end
			pdd:SetList(ddvals, b)
			pdd.frame:Show()
			eb.frame:Hide()
		else
			pdd.frame:Hide()
			eb.frame:Hide()
		end
		MT.builder["add" .. row] = (key == "zero") and "" or key
		MT.builder["addp" .. row] = nil
	end

	local function addpvaluechanged(value, row) MT.builder["addp" .. row] = (value == "zero") and "" or value end
	local mtbfaddp1 = AceGUI:Create("Dropdown")
	local mtbfadde1 = AceGUI:Create("EditBox")
	local mtbfaddp2 = AceGUI:Create("Dropdown")
	local mtbfadde2 = AceGUI:Create("EditBox")
	local mtbfaddp3 = AceGUI:Create("Dropdown")
	local mtbfadde3 = AceGUI:Create("EditBox")
	local mtbfaddp4 = AceGUI:Create("Dropdown")
	local mtbfadde4 = AceGUI:Create("EditBox")
	local mtbfaddp5 = AceGUI:Create("Dropdown")
	local mtbfadde5 = AceGUI:Create("EditBox")
	local mtbfadd1 = AceGUI:Create("Dropdown")
	mtbfadd1.frame:SetParent(mtbf)
	mtbfadd1:SetList(ddother, ddotherorder)
	mtbfadd1:SetWidth(200)
	mtbfadd1:SetLabel(L["Additional conditions"])
	mtbfadd1.label:SetTextColor(1, 1, 1, 1)
	mtbfadd1:SetCallback("OnValueChanged", function(info, name, key) addvaluechanged(key, 1, mtbfaddp1, mtbfadde1) end)
	mtbfadd1:SetPoint("TOPLEFT", mtbftopts1.frame, "BOTTOMLEFT", 0, -5)
	
	mtbfaddp1.frame:SetParent(mtbf)
	mtbfaddp1:SetWidth(150)
	mtbfaddp1:SetCallback("OnValueChanged", function(info, name, key) addpvaluechanged(key, 1) end)
	mtbfaddp1:SetPoint("LEFT", mtbfadd1.frame, "RIGHT", 3, -9)
	mtbfaddp1.frame:Hide()

	mtbfadde1.frame:SetParent(mtbf)
	mtbfadde1:SetWidth(150)
	mtbfadde1:SetCallback("OnEnterPressed", function(info, name, value) addpvaluechanged(value, 1) end)
	mtbfadde1:SetPoint("LEFT", mtbfadd1.frame, "RIGHT", 3, -9)
	mtbfadde1.frame:Hide()
	
	local mtbfadd2 = AceGUI:Create("Dropdown")
	mtbfadd2.frame:SetParent(mtbf)
	mtbfadd2:SetList(ddother, ddotherorder)
	mtbfadd2:SetWidth(200)
	mtbfadd2:SetCallback("OnValueChanged", function(info, name, key) addvaluechanged(key, 2, mtbfaddp2, mtbfadde2) end)
	mtbfadd2:SetPoint("TOPLEFT", mtbfadd1.frame, "BOTTOMLEFT", 0, -10)
	
	mtbfaddp2.frame:SetParent(mtbf)
	mtbfaddp2:SetWidth(150)
	mtbfaddp2:SetCallback("OnValueChanged", function(info, name, key) addpvaluechanged(key, 2) end)
	mtbfaddp2:SetPoint("LEFT", mtbfadd2.frame, "RIGHT", 3, 0)
	mtbfaddp2.frame:Hide()

	mtbfadde2.frame:SetParent(mtbf)
	mtbfadde2:SetWidth(150)
	mtbfadde2:SetCallback("OnEnterPressed", function(info, name, value) addpvaluechanged(value, 2) end)
	mtbfadde2:SetPoint("LEFT", mtbfadd2.frame, "RIGHT", 3, 0)
	mtbfadde2.frame:Hide()
	
	local mtbfadd3 = AceGUI:Create("Dropdown")
	mtbfadd3.frame:SetParent(mtbf)
	mtbfadd3:SetList(ddother, ddotherorder)
	mtbfadd3:SetWidth(200)
	mtbfadd3:SetCallback("OnValueChanged", function(info, name, key) addvaluechanged(key, 3, mtbfaddp3, mtbfadde3) end)
	mtbfadd3:SetPoint("TOPLEFT", mtbfadd2.frame, "BOTTOMLEFT", 0, -10)
	
	mtbfaddp3.frame:SetParent(mtbf)
	mtbfaddp3:SetWidth(150)
	mtbfaddp3:SetCallback("OnValueChanged", function(info, name, key) addpvaluechanged(key, 3) end)
	mtbfaddp3:SetPoint("LEFT", mtbfadd3.frame, "RIGHT", 3, 0)
	mtbfaddp3.frame:Hide()

	mtbfadde3.frame:SetParent(mtbf)
	mtbfadde3:SetWidth(150)
	mtbfadde3:SetCallback("OnEnterPressed", function(info, name, value) addpvaluechanged(value, 3) end)
	mtbfadde3:SetPoint("LEFT", mtbfadd2.frame, "RIGHT", 3, 0)
	mtbfadde3.frame:Hide()
	
	local mtbfadd4 = AceGUI:Create("Dropdown")
	mtbfadd4.frame:SetParent(mtbf)
	mtbfadd4:SetList(ddother, ddotherorder)
	mtbfadd4:SetWidth(200)
	mtbfadd4:SetCallback("OnValueChanged", function(info, name, key) addvaluechanged(key, 4, mtbfaddp4, mtbfadde4) end)
	mtbfadd4:SetPoint("TOPLEFT", mtbfadd3.frame, "BOTTOMLEFT", 0, -10)
	
	mtbfaddp4.frame:SetParent(mtbf)
	mtbfaddp4:SetWidth(150)
	mtbfaddp4:SetCallback("OnValueChanged", function(info, name, key) addpvaluechanged(key, 4) end)
	mtbfaddp4:SetPoint("LEFT", mtbfadd4.frame, "RIGHT", 3, 0)
	mtbfaddp4.frame:Hide()

	mtbfadde4.frame:SetParent(mtbf)
	mtbfadde4:SetWidth(150)
	mtbfadde4:SetCallback("OnEnterPressed", function(info, name, value) addpvaluechanged(value, 4) end)
	mtbfadde4:SetPoint("LEFT", mtbfadd4.frame, "RIGHT", 3, 0)
	mtbfadde4.frame:Hide()
	
	local mtbfadd5 = AceGUI:Create("Dropdown")
	mtbfadd5.frame:SetParent(mtbf)
	mtbfadd5:SetList(ddother, ddotherorder)
	mtbfadd5:SetWidth(200)
	mtbfadd5:SetCallback("OnValueChanged", function(info, name, key) addvaluechanged(key, 5, mtbfaddp5, mtbfadde5) end)
	mtbfadd5:SetPoint("TOPLEFT", mtbfadd4.frame, "BOTTOMLEFT", 0, -10)
	
	mtbfaddp5.frame:SetParent(mtbf)
	mtbfaddp5:SetWidth(150)
	mtbfaddp5:SetCallback("OnValueChanged", function(info, name, key) addpvaluechanged(key, 5) end)
	mtbfaddp5:SetPoint("LEFT", mtbfadd5.frame, "RIGHT", 3,0)
	mtbfaddp5.frame:Hide()

	mtbfadde5.frame:SetParent(mtbf)
	mtbfadde5:SetWidth(150)
	mtbfadde5:SetCallback("OnEnterPressed", function(info, name, value) addpvaluechanged(value, 5) end)
	mtbfadde5:SetPoint("LEFT", mtbfadd5.frame, "RIGHT", 3, 0)
	mtbfadde5.frame:Hide()
	
	local mtbcancel = CreateFrame("Button", "MacroToolkitBuilderCancel", mtbf, "UIPanelButtonTemplate")
	mtbcancel:SetText(_G.CANCEL)
	mtbcancel:SetSize(80, 22)
	mtbcancel:SetPoint("BOTTOMRIGHT", -5, 4)
	mtbcancel:SetScript("OnClick", HideParentPanel)

	local mtbinsert = CreateFrame("Button", "MacroToolkitBuilderInsert", mtbf, "UIPanelButtonTemplate")
	mtbinsert:SetText(_G.KEY_INSERT)
	mtbinsert:SetSize(80, 22)
	mtbinsert:SetPoint("RIGHT", mtbcancel, "LEFT")
	mtbinsert:SetScript("OnClick", function() MT:BuildAndInsert() end)

	mtbf:SetScript("OnShow",
		function()
			mtbf:SetPoint("BOTTOMLEFT", MT.db.profile.x, MT.db.profile.y)
			MT.builder = {}
			local ddobjs1 = {mtbftarget, mtbftopts1, mtbftopts2, mtbftopts3, mtbftopts4, mtbfadd1, mtbfadd2, mtbfadd3, mtbfadd4, mtbfadd5}
			local ddobjs2 = {mtbfaddp1, mtbfaddp2, mtbfaddp3, mtbfaddp4, mtbfaddp5}
			local ebobjs = {mtbfother, mtbfadde1, mtbfadde2, mtbfadde3, mtbfadde4, mtbfadde5}
			for _, dd in ipairs(ddobjs1) do dd:SetValue("zero") end
			for _, dd in ipairs(ddobjs2) do dd:SetValue("zero"); dd.frame:Hide() end
			for _, eb in ipairs(ebobjs) do eb:SetText(""); eb.frame:Hide() end
		end)
	
	mtbf:SetScript("OnHide", function() ShowUIPanel(MacroToolkitFrame) end)
end